@@ -1,25 +1,23 @@
 mod metadata;
-mod pep639_glob;
 
 use crate::metadata::{PyProjectToml, ValidationError};
-use crate::pep639_glob::Pep639GlobError;
 use flate2::write::GzEncoder;
 use flate2::Compression;
 use fs_err::File;
-use glob::{GlobError, PatternError};
-use globset::{Glob, GlobSetBuilder};
+use globset::GlobSetBuilder;
 use itertools::Itertools;
 use sha2::{Digest, Sha256};
 use std::fs::FileType;
 use std::io::{BufReader, Cursor, Read, Write};
 use std::path::{Path, PathBuf, StripPrefixError};
 use std::{io, mem};
-use tar::{EntryType, Header};
+use tar::{Builder, EntryType, Header};
 use thiserror::Error;
 use tracing::{debug, trace};
 use uv_distribution_filename::{SourceDistExtension, SourceDistFilename, WheelFilename};
 use uv_fs::Simplified;
-use walkdir::WalkDir;
+use uv_globfilter::{parse_portable_glob, GlobDirFilter, PortableGlobError};
+use walkdir::{DirEntry, WalkDir};
 use zip::{CompressionMethod, ZipWriter};
 
 #[derive(Debug, Error)]
@@ -30,16 +28,26 @@ pub enum Error {
     Toml(#[from] toml::de::Error),
     #[error("Invalid pyproject.toml")]
     Validation(#[from] ValidationError),
-    #[error("Invalid `project.license-files` glob expression: `{0}`")]
-    Pep639Glob(String, #[source] Pep639GlobError),
-    #[error("The `project.license-files` entry is not a valid glob pattern: `{0}`")]
-    Pattern(String, #[source] PatternError),
-    /// [`GlobError`] is a wrapped io error.
-    #[error(transparent)]
-    Glob(#[from] GlobError),
+    #[error("Unsupported glob expression in: `{field}`")]
+    PortableGlob {
+        field: String,
+        #[source]
+        source: PortableGlobError,
+    },
+    /// <https://github.com/BurntSushi/ripgrep/discussions/2927>
+    #[error("Glob expressions caused to large regex in: `{field}`")]
+    GlobSetTooLarge {
+        field: String,
+        #[source]
+        source: globset::Error,
+    },
     /// [`globset::Error`] shows the glob that failed to parse.
-    #[error(transparent)]
-    GlobSet(#[from] globset::Error),
+    #[error("Unsupported glob expression in: `{field}`")]
+    GlobSet {
+        field: String,
+        #[source]
+        err: globset::Error,
+    },
     #[error("Failed to walk source tree: `{}`", root.user_display())]
     WalkDir {
         root: PathBuf,
@@ -58,6 +66,8 @@ pub enum Error {
     Csv(#[from] csv::Error),
     #[error("Expected a Python module with an `__init__.py` at: `{}`", _0.user_display())]
     MissingModule(PathBuf),
+    #[error("Absolute module root is not allowed: `{}`", _0.display())]
+    AbsoluteModuleRoot(PathBuf),
     #[error("Inconsistent metadata between prepare and build step: `{0}`")]
     InconsistentSteps(&'static str),
     #[error("Failed to write to {}", _0.user_display())]
@@ -284,11 +294,29 @@ fn write_hashed(
     })
 }
 
+/// TODO(konsti): Wire this up with actual settings and remove this struct.
+///
+/// Which files to include in the wheel
+pub struct WheelSettings {
+    /// The directory that contains the module directory, usually `src`, or an empty path when
+    /// using the flat layout over the src layout.
+    module_root: PathBuf,
+}
+
+impl Default for WheelSettings {
+    fn default() -> Self {
+        Self {
+            module_root: PathBuf::from("src"),
+        }
+    }
+}
+
 /// Build a wheel from the source tree and place it in the output directory.
 pub fn build_wheel(
     source_tree: &Path,
     wheel_dir: &Path,
     metadata_directory: Option<&Path>,
+    wheel_settings: WheelSettings,
     uv_version: &str,
 ) -> Result<WheelFilename, Error> {
     let contents = fs_err::read_to_string(source_tree.join("pyproject.toml"))?;
@@ -311,7 +339,10 @@ pub fn build_wheel(
     let mut wheel_writer = ZipDirectoryWriter::new_wheel(File::create(&wheel_path)?);
 
     debug!("Adding content files to {}", wheel_path.user_display());
-    let strip_root = source_tree.join("src");
+    if wheel_settings.module_root.is_absolute() {
+        return Err(Error::AbsoluteModuleRoot(wheel_settings.module_root));
+    }
+    let strip_root = source_tree.join(wheel_settings.module_root);
     let module_root = strip_root.join(pyproject_toml.name().as_dist_info_name().as_ref());
     if !module_root.join("__init__.py").is_file() {
         return Err(Error::MissingModule(module_root));
@@ -322,10 +353,16 @@ pub fn build_wheel(
             err,
         })?;
 
-        let relative_path = entry.path().strip_prefix(&strip_root)?;
+        let relative_path = entry
+            .path()
+            .strip_prefix(&strip_root)
+            .expect("walkdir starts with root");
         let relative_path_str = relative_path
             .to_str()
             .ok_or_else(|| Error::NotUtf8Path(relative_path.to_path_buf()))?;
+
+        debug!("Adding to wheel: `{relative_path_str}`");
+
         if entry.file_type().is_dir() {
             wheel_writer.write_directory(relative_path_str)?;
         } else if entry.file_type().is_file() {
@@ -354,10 +391,52 @@ pub fn build_wheel(
     Ok(filename)
 }
 
+/// TODO(konsti): Wire this up with actual settings and remove this struct.
+///
+/// To select which files to include in the source distribution, we first add the includes, then
+/// remove the excludes from that.
+pub struct SourceDistSettings {
+    /// Glob expressions which files and directories to include in the source distribution.
+    ///
+    /// Includes are anchored, which means that `pyproject.toml` includes only
+    /// `<project root>/pyproject.toml`. Use for example `assets/**/sample.csv` to include for all
+    /// `sample.csv` files in `<project root>/assets` or any child directory. To recursively include
+    /// all files under a directory, use a `/**` suffix, e.g. `src/**`. For performance and
+    /// reproducibility, avoid unanchored matches such as `**/sample.csv`.
+    ///
+    /// The glob syntax is the reduced portable glob from
+    /// [PEP 639](https://peps.python.org/pep-0639/#add-license-FILES-key).
+    include: Vec<String>,
+    /// Glob expressions which files and directories to exclude from the previous source
+    /// distribution includes.
+    ///
+    /// Excludes are not anchored, which means that `__pycache__` excludes all directories named
+    /// `__pycache__` and it's children anywhere. To anchor a directory, use a `/` prefix, e.g.,
+    /// `/dist` will exclude only `<project root>/dist`.
+    ///
+    /// The glob syntax is the reduced portable glob from
+    /// [PEP 639](https://peps.python.org/pep-0639/#add-license-FILES-key).
+    exclude: Vec<String>,
+}
+
+impl Default for SourceDistSettings {
+    fn default() -> Self {
+        Self {
+            include: vec!["src/**".to_string(), "pyproject.toml".to_string()],
+            exclude: vec![
+                "__pycache__".to_string(),
+                "*.pyc".to_string(),
+                "*.pyo".to_string(),
+            ],
+        }
+    }
+}
+
 /// Build a source distribution from the source tree and place it in the output directory.
 pub fn build_source_dist(
     source_tree: &Path,
     source_dist_directory: &Path,
+    settings: SourceDistSettings,
     uv_version: &str,
 ) -> Result<SourceDistFilename, Error> {
     let contents = fs_err::read_to_string(source_tree.join("pyproject.toml"))?;
@@ -392,80 +471,74 @@ pub fn build_source_dist(
     )
     .map_err(|err| Error::TarWrite(source_dist_path.clone(), err))?;
 
-    let includes = ["src/**/*", "pyproject.toml"];
-    let mut include_builder = GlobSetBuilder::new();
-    for include in includes {
-        include_builder.add(Glob::new(include)?);
+    let mut include_globs = Vec::new();
+    for include in settings.include {
+        let glob = parse_portable_glob(&include).map_err(|err| Error::PortableGlob {
+            field: "tool.uv.source-dist.include".to_string(),
+            source: err,
+        })?;
+        include_globs.push(glob.clone());
     }
-    let include_matcher = include_builder.build()?;
+    let include_matcher =
+        GlobDirFilter::from_globs(&include_globs).map_err(|err| Error::GlobSetTooLarge {
+            field: "tool.uv.source-dist.include".to_string(),
+            source: err,
+        })?;
 
-    let excludes = ["__pycache__", "*.pyc", "*.pyo"];
     let mut exclude_builder = GlobSetBuilder::new();
-    for exclude in excludes {
-        exclude_builder.add(Glob::new(exclude)?);
+    for exclude in settings.exclude {
+        // Excludes are unanchored
+        let exclude = if let Some(exclude) = exclude.strip_prefix("/") {
+            exclude.to_string()
+        } else {
+            format!("**/{exclude}").to_string()
+        };
+        let glob = parse_portable_glob(&exclude).map_err(|err| Error::PortableGlob {
+            field: "tool.uv.source-dist.exclude".to_string(),
+            source: err,
+        })?;
+        exclude_builder.add(glob);
     }
-    let exclude_matcher = exclude_builder.build()?;
+    let exclude_matcher = exclude_builder
+        .build()
+        .map_err(|err| Error::GlobSetTooLarge {
+            field: "tool.uv.source-dist.exclude".to_string(),
+            source: err,
+        })?;
 
     // TODO(konsti): Add files linked by pyproject.toml
 
-    for file in WalkDir::new(source_tree).into_iter().filter_entry(|dir| {
-        let relative = dir
-            .path()
-            .strip_prefix(source_tree)
-            .expect("walkdir starts with root");
-        // TODO(konsti): Also check that we're matching at least a prefix of an include matcher.
-        !exclude_matcher.is_match(relative)
-    }) {
-        let entry = file.map_err(|err| Error::WalkDir {
-            root: source_tree.to_path_buf(),
-            err,
-        })?;
+    for entry in WalkDir::new(source_tree).into_iter().filter_entry(|entry| {
+        // TODO(konsti): This should be prettier.
         let relative = entry
             .path()
             .strip_prefix(source_tree)
             .expect("walkdir starts with root");
-        if !include_matcher.is_match(relative) {
+
+        // Fast path: Don't descend into a directory that can't be included. This is the most
+        // important performance optimization, it avoids descending into directories such as
+        // `.venv`. While walkdir is generally cheap, we still avoid traversing large data
+        // directories that often exist on the top level of a project. This is especially noticeable
+        // on network file systems with high latencies per operation (while contiguous reading may
+        // still be fast).
+        include_matcher.match_directory(relative) && !exclude_matcher.is_match(relative)
+    }) {
+        let entry = entry.map_err(|err| Error::WalkDir {
+            root: source_tree.to_path_buf(),
+            err,
+        })?;
+        // TODO(konsti): This should be prettier.
+        let relative = entry
+            .path()
+            .strip_prefix(source_tree)
+            .expect("walkdir starts with root");
+
+        if !include_matcher.match_path(relative) || exclude_matcher.is_match(relative) {
             trace!("Excluding {}", relative.user_display());
             continue;
-        }
-        debug!("Including {}", relative.user_display());
+        };
 
-        let metadata = fs_err::metadata(entry.path())?;
-        let mut header = Header::new_gnu();
-        #[cfg(unix)]
-        {
-            header.set_mode(std::os::unix::fs::MetadataExt::mode(&metadata));
-        }
-        #[cfg(not(unix))]
-        {
-            header.set_mode(0o644);
-        }
-
-        if entry.file_type().is_dir() {
-            header.set_entry_type(EntryType::Directory);
-            header
-                .set_path(Path::new(&top_level).join(relative))
-                .map_err(|err| Error::TarWrite(source_dist_path.clone(), err))?;
-            header.set_size(0);
-            header.set_cksum();
-            tar.append(&header, io::empty())
-                .map_err(|err| Error::TarWrite(source_dist_path.clone(), err))?;
-            continue;
-        } else if entry.file_type().is_file() {
-            header.set_size(metadata.len());
-            header.set_cksum();
-            tar.append_data(
-                &mut header,
-                Path::new(&top_level).join(relative),
-                BufReader::new(File::open(entry.path())?),
-            )
-            .map_err(|err| Error::TarWrite(source_dist_path.clone(), err))?;
-        } else {
-            return Err(Error::UnsupportedFileType(
-                relative.to_path_buf(),
-                entry.file_type(),
-            ));
-        }
+        add_source_dist_entry(&mut tar, &entry, &top_level, &source_dist_path, relative)?;
     }
 
     tar.finish()
@@ -474,6 +547,55 @@ pub fn build_source_dist(
     Ok(filename)
 }
 
+/// Add a file or a directory to a source distribution.
+fn add_source_dist_entry(
+    tar: &mut Builder<GzEncoder<File>>,
+    entry: &DirEntry,
+    top_level: &str,
+    source_dist_path: &Path,
+    relative: &Path,
+) -> Result<(), Error> {
+    debug!("Including {}", relative.user_display());
+
+    let metadata = fs_err::metadata(entry.path())?;
+    let mut header = Header::new_gnu();
+    #[cfg(unix)]
+    {
+        header.set_mode(std::os::unix::fs::MetadataExt::mode(&metadata));
+    }
+    #[cfg(not(unix))]
+    {
+        header.set_mode(0o644);
+    }
+
+    if entry.file_type().is_dir() {
+        header.set_entry_type(EntryType::Directory);
+        header
+            .set_path(Path::new(&top_level).join(relative))
+            .map_err(|err| Error::TarWrite(source_dist_path.to_path_buf(), err))?;
+        header.set_size(0);
+        header.set_cksum();
+        tar.append(&header, io::empty())
+            .map_err(|err| Error::TarWrite(source_dist_path.to_path_buf(), err))?;
+        Ok(())
+    } else if entry.file_type().is_file() {
+        header.set_size(metadata.len());
+        header.set_cksum();
+        tar.append_data(
+            &mut header,
+            Path::new(&top_level).join(relative),
+            BufReader::new(File::open(entry.path())?),
+        )
+        .map_err(|err| Error::TarWrite(source_dist_path.to_path_buf(), err))?;
+        Ok(())
+    } else {
+        Err(Error::UnsupportedFileType(
+            relative.to_path_buf(),
+            entry.file_type(),
+        ))
+    }
+}
+
 /// Write the dist-info directory to the output directory without building the wheel.
 pub fn metadata(
     source_tree: &Path,
