@@ -10,9 +10,10 @@ use rustc_hash::FxHashMap;
 use tracing::trace;
 
 use uv_distribution_types::{
-    BuiltDist, IndexCapabilities, IndexLocations, IndexUrl, InstalledDist, SourceDist,
+    BuiltDist, DerivationChain, IndexCapabilities, IndexLocations, IndexUrl, InstalledDist,
+    SourceDist,
 };
-use uv_normalize::PackageName;
+use uv_normalize::{ExtraName, PackageName};
 use uv_pep440::{LocalVersionSlice, Version};
 use uv_static::EnvVars;
 
@@ -32,6 +33,9 @@ pub enum ResolveError {
     #[error(transparent)]
     Client(#[from] uv_client::Error),
 
+    #[error(transparent)]
+    Distribution(#[from] uv_distribution::Error),
+
     #[error("The channel closed unexpectedly")]
     ChannelClosed,
 
@@ -41,6 +45,13 @@ pub enum ResolveError {
     #[error("Attempted to wait on an unregistered task: `{_0}`")]
     UnregisteredTask(String),
 
+    #[error("Found conflicting extra `{extra}` unconditionally enabled in `{requirement}`")]
+    ConflictingExtra {
+        // Boxed because `Requirement` is large.
+        requirement: Box<uv_pypi_types::Requirement>,
+        extra: ExtraName,
+    },
+
     #[error("Overrides contain conflicting URLs for package `{0}`:\n- {1}\n- {2}")]
     ConflictingOverrideUrls(PackageName, String, String),
 
@@ -87,20 +98,36 @@ pub enum ResolveError {
     ParsedUrl(#[from] uv_pypi_types::ParsedUrlError),
 
     #[error("Failed to download `{0}`")]
-    Fetch(Box<BuiltDist>, #[source] uv_distribution::Error),
+    Download(
+        Box<BuiltDist>,
+        DerivationChain,
+        #[source] Arc<uv_distribution::Error>,
+    ),
 
     #[error("Failed to download and build `{0}`")]
-    FetchAndBuild(Box<SourceDist>, #[source] uv_distribution::Error),
+    DownloadAndBuild(
+        Box<SourceDist>,
+        DerivationChain,
+        #[source] Arc<uv_distribution::Error>,
+    ),
 
     #[error("Failed to read `{0}`")]
-    Read(Box<BuiltDist>, #[source] uv_distribution::Error),
+    Read(
+        Box<BuiltDist>,
+        DerivationChain,
+        #[source] Arc<uv_distribution::Error>,
+    ),
 
     // TODO(zanieb): Use `thiserror` in `InstalledDist` so we can avoid chaining `anyhow`
     #[error("Failed to read metadata from installed package `{0}`")]
-    ReadInstalled(Box<InstalledDist>, #[source] anyhow::Error),
+    ReadInstalled(Box<InstalledDist>, DerivationChain, #[source] anyhow::Error),
 
     #[error("Failed to build `{0}`")]
-    Build(Box<SourceDist>, #[source] uv_distribution::Error),
+    Build(
+        Box<SourceDist>,
+        DerivationChain,
+        #[source] Arc<uv_distribution::Error>,
+    ),
 
     #[error(transparent)]
     NoSolution(#[from] NoSolutionError),
