@@ -14,17 +14,24 @@ pub use prerelease::PrereleaseMode;
 pub use python_requirement::PythonRequirement;
 pub use requires_python::{RequiresPython, RequiresPythonRange};
 pub use resolution::{
-    AnnotationStyle, ConflictingDistributionError, DisplayResolutionGraph, ResolutionGraph,
+    AnnotationStyle, ConflictingDistributionError, DisplayResolutionGraph, ResolverOutput,
 };
 pub use resolution_mode::ResolutionMode;
 pub use resolver::{
-    BuildId, DefaultResolverProvider, InMemoryIndex, MetadataResponse, PackageVersionsResult,
-    Reporter as ResolverReporter, Resolver, ResolverEnvironment, ResolverProvider,
-    VersionsResponse, WheelMetadataResult,
+    BuildId, DefaultResolverProvider, DerivationChainBuilder, InMemoryIndex, MetadataResponse,
+    PackageVersionsResult, Reporter as ResolverReporter, Resolver, ResolverEnvironment,
+    ResolverProvider, VersionsResponse, WheelMetadataResult,
 };
 pub use version_map::VersionMap;
 pub use yanks::AllowedYanks;
 
+/// A custom `HashSet` using `hashbrown`.
+///
+/// We use `hashbrown` instead of `std` to get access to its `Equivalent`
+/// trait. This lets use store things like `ConflictItem`, but refer to it via
+/// `ConflictItemRef`. i.e., We can avoid allocs on lookups.
+type FxHashbrownSet<T> = hashbrown::HashSet<T, rustc_hash::FxBuildHasher>;
+
 mod bare;
 mod candidate_selector;
 
