@@ -1,38 +1,30 @@
-use itertools::Itertools;
 use pubgrub::Range;
 use std::cmp::Ordering;
 use std::collections::Bound;
 use std::ops::Deref;
 
 use uv_distribution_filename::WheelFilename;
-use uv_pep440::{Version, VersionSpecifier, VersionSpecifiers};
+use uv_pep440::{release_specifiers_to_ranges, Version, VersionSpecifier, VersionSpecifiers};
 use uv_pep508::{MarkerExpression, MarkerTree, MarkerValueVersion};
-use uv_pubgrub::PubGrubSpecifier;
-
-#[derive(thiserror::Error, Debug)]
-pub enum RequiresPythonError {
-    #[error(transparent)]
-    PubGrub(#[from] crate::pubgrub::PubGrubSpecifierError),
-}
 
 /// The `Requires-Python` requirement specifier.
 ///
-/// We treat `Requires-Python` as a lower bound. For example, if the requirement expresses
-/// `>=3.8, <4`, we treat it as `>=3.8`. `Requires-Python` itself was intended to enable
-/// packages to drop support for older versions of Python without breaking installations on
-/// those versions, and packages cannot know whether they are compatible with future, unreleased
-/// versions of Python.
-///
 /// See: <https://packaging.python.org/en/latest/guides/dropping-older-python-versions/>
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct RequiresPython {
     /// The supported Python versions as provides by the user, usually through the `requires-python`
     /// field in `pyproject.toml`.
     ///
-    /// For a workspace, it's the union of all `requires-python` values in the workspace. If no
-    /// bound was provided by the user, it's greater equal the current Python version.
+    /// For a workspace, it's the intersection of all `requires-python` values in the workspace. If
+    /// no bound was provided by the user, it's greater equal the current Python version.
+    ///
+    /// The specifiers remain static over the lifetime of the workspace, such that they
+    /// represent the initial Python version constraints.
     specifiers: VersionSpecifiers,
-    /// The lower and upper bounds of `specifiers`.
+    /// The lower and upper bounds of the given specifiers.
+    ///
+    /// The range may be narrowed over the course of dependency resolution as the resolver
+    /// investigates environments with stricter Python version constraints.
     range: RequiresPythonRange,
 }
 
@@ -52,15 +44,15 @@ impl RequiresPython {
     }
 
     /// Returns a [`RequiresPython`] from a version specifier.
-    pub fn from_specifiers(specifiers: &VersionSpecifiers) -> Result<Self, RequiresPythonError> {
-        let (lower_bound, upper_bound) = PubGrubSpecifier::from_release_specifiers(specifiers)?
+    pub fn from_specifiers(specifiers: &VersionSpecifiers) -> Self {
+        let (lower_bound, upper_bound) = release_specifiers_to_ranges(specifiers.clone())
             .bounding_range()
             .map(|(lower_bound, upper_bound)| (lower_bound.cloned(), upper_bound.cloned()))
             .unwrap_or((Bound::Unbounded, Bound::Unbounded));
-        Ok(Self {
+        Self {
             specifiers: specifiers.clone(),
             range: RequiresPythonRange(LowerBound(lower_bound), UpperBound(upper_bound)),
-        })
+        }
     }
 
     /// Returns a [`RequiresPython`] to express the intersection of the given version specifiers.
@@ -68,23 +60,19 @@ impl RequiresPython {
     /// For example, given `>=3.8` and `>=3.9`, this would return `>=3.9`.
     pub fn intersection<'a>(
         specifiers: impl Iterator<Item = &'a VersionSpecifiers>,
-    ) -> Result<Option<Self>, RequiresPythonError> {
+    ) -> Option<Self> {
         // Convert to PubGrub range and perform an intersection.
         let range = specifiers
             .into_iter()
-            .map(PubGrubSpecifier::from_release_specifiers)
-            .fold_ok(None, |range: Option<Range<Version>>, requires_python| {
+            .map(|specifier| release_specifiers_to_ranges(specifier.clone()))
+            .fold(None, |range: Option<Range<Version>>, requires_python| {
                 if let Some(range) = range {
-                    Some(range.intersection(&requires_python.into()))
+                    Some(range.intersection(&requires_python))
                 } else {
-                    Some(requires_python.into())
+                    Some(requires_python)
                 }
             })?;
 
-        let Some(range) = range else {
-            return Ok(None);
-        };
-
         // Extract the bounds.
         let (lower_bound, upper_bound) = range
             .bounding_range()
@@ -99,13 +87,13 @@ impl RequiresPython {
         // Convert back to PEP 440 specifiers.
         let specifiers = VersionSpecifiers::from_release_only_bounds(range.iter());
 
-        Ok(Some(Self {
+        Some(Self {
             specifiers,
             range: RequiresPythonRange(lower_bound, upper_bound),
-        }))
+        })
     }
 
-    /// Narrow the [`RequiresPython`] to the given version, if it's stricter than the current target.
+    /// Narrow the [`RequiresPython`] by computing the intersection with the given range.
     pub fn narrow(&self, range: &RequiresPythonRange) -> Option<Self> {
         let lower = if range.0 >= self.range.0 {
             Some(&range.0)
@@ -117,6 +105,8 @@ impl RequiresPython {
         } else {
             None
         };
+        // TODO(charlie): Consider re-computing the specifiers (or removing them entirely in favor
+        // of tracking the range). After narrowing, the specifiers and range may be out of sync.
         match (lower, upper) {
             (Some(lower), Some(upper)) => Some(Self {
                 specifiers: self.specifiers.clone(),
@@ -225,26 +215,40 @@ impl RequiresPython {
     }
 
     /// Returns `true` if the `Requires-Python` is compatible with the given version.
+    ///
+    /// N.B. This operation should primarily be used when evaluating compatibility of Python
+    /// versions against the user's own project. For example, if the user defines a
+    /// `requires-python` in a `pyproject.toml`, this operation could be used to determine whether
+    /// a given Python interpreter is compatible with the user's project.
     pub fn contains(&self, version: &Version) -> bool {
         let version = version.only_release();
         self.specifiers.contains(&version)
     }
 
-    /// Returns `true` if the `Requires-Python` is compatible with the given version specifiers.
+    /// Returns `true` if the `Requires-Python` is contained by the given version specifiers.
+    ///
+    /// In this context, we treat `Requires-Python` as a lower bound. For example, if the
+    /// requirement expresses `>=3.8, <4`, we treat it as `>=3.8`. `Requires-Python` itself was
+    /// intended to enable packages to drop support for older versions of Python without breaking
+    /// installations on those versions, and packages cannot know whether they are compatible with
+    /// future, unreleased versions of Python.
+    ///
+    /// The specifiers are considered to "contain" the `Requires-Python` if the specifiers are
+    /// compatible with all versions in the `Requires-Python` range (i.e., have a _lower_ lower
+    /// bound).
     ///
     /// For example, if the `Requires-Python` is `>=3.8`, then `>=3.7` would be considered
     /// compatible, since all versions in the `Requires-Python` range are also covered by the
     /// provided range. However, `>=3.9` would not be considered compatible, as the
     /// `Requires-Python` includes Python 3.8, but `>=3.9` does not.
+    ///
+    /// N.B. This operation should primarily be used when evaluating the compatibility of a
+    /// project's `Requires-Python` specifier against a dependency's `Requires-Python` specifier.
     pub fn is_contained_by(&self, target: &VersionSpecifiers) -> bool {
-        let Ok(target) = PubGrubSpecifier::from_release_specifiers(target) else {
-            return false;
-        };
-        let target = target
-            .iter()
-            .next()
-            .map(|(lower, _)| lower)
-            .unwrap_or(&Bound::Unbounded);
+        let target = release_specifiers_to_ranges(target.clone())
+            .bounding_range()
+            .map(|bounding_range| bounding_range.0.cloned())
+            .unwrap_or(Bound::Unbounded);
 
         // We want, e.g., `self.range.lower()` to be `>=3.8` and `target` to be `>=3.7`.
         //
@@ -485,8 +489,7 @@ impl serde::Serialize for RequiresPython {
 impl<'de> serde::Deserialize<'de> for RequiresPython {
     fn deserialize<D: serde::Deserializer<'de>>(deserializer: D) -> Result<Self, D::Error> {
         let specifiers = VersionSpecifiers::deserialize(deserializer)?;
-        let (lower_bound, upper_bound) = PubGrubSpecifier::from_release_specifiers(&specifiers)
-            .map_err(serde::de::Error::custom)?
+        let (lower_bound, upper_bound) = release_specifiers_to_ranges(specifiers.clone())
             .bounding_range()
             .map(|(lower_bound, upper_bound)| (lower_bound.cloned(), upper_bound.cloned()))
             .unwrap_or((Bound::Unbounded, Bound::Unbounded));
