@@ -6,7 +6,7 @@ use indicatif::{MultiProgress, ProgressBar, ProgressStyle};
 use owo_colors::OwoColorize;
 use rustc_hash::FxHashMap;
 use url::Url;
-
+use uv_cache::Removal;
 use uv_distribution_types::{
     BuildableSource, CachedDist, DistributionMetadata, Name, SourceDist, VersionOrUrlRef,
 };
@@ -528,6 +528,66 @@ impl uv_publish::Reporter for PublishReporter {
     }
 }
 
+#[derive(Debug)]
+pub(crate) struct CleaningDirectoryReporter {
+    bar: ProgressBar,
+}
+
+impl CleaningDirectoryReporter {
+    /// Initialize a [`CleaningDirectoryReporter`] for cleaning the cache directory.
+    pub(crate) fn new(printer: Printer, max: usize) -> Self {
+        let bar = ProgressBar::with_draw_target(Some(max as u64), printer.target());
+        bar.set_style(
+            ProgressStyle::with_template("{prefix} [{bar:20}] {percent}%")
+                .unwrap()
+                .progress_chars("=> "),
+        );
+        bar.set_prefix(format!("{}", "Cleaning".bold().cyan()));
+        Self { bar }
+    }
+}
+
+impl uv_cache::CleanReporter for CleaningDirectoryReporter {
+    fn on_clean(&self) {
+        self.bar.inc(1);
+    }
+
+    fn on_complete(&self) {
+        self.bar.finish_and_clear();
+    }
+}
+
+#[derive(Debug)]
+pub(crate) struct CleaningPackageReporter {
+    bar: ProgressBar,
+}
+
+impl CleaningPackageReporter {
+    /// Initialize a [`CleaningPackageReporter`] for cleaning packages from the cache.
+    pub(crate) fn new(printer: Printer, max: usize) -> Self {
+        let bar = ProgressBar::with_draw_target(Some(max as u64), printer.target());
+        bar.set_style(
+            ProgressStyle::with_template("{prefix} [{bar:20}] {pos}/{len}{msg}")
+                .unwrap()
+                .progress_chars("=> "),
+        );
+        bar.set_prefix(format!("{}", "Cleaning".bold().cyan()));
+        Self { bar }
+    }
+
+    pub(crate) fn on_clean(&self, package: &str, removal: &Removal) {
+        self.bar.inc(1);
+        self.bar.set_message(format!(
+            ": {}, {} files {} folders removed",
+            package, removal.num_files, removal.num_dirs,
+        ));
+    }
+
+    pub(crate) fn on_complete(&self) {
+        self.bar.finish_and_clear();
+    }
+}
+
 /// Like [`std::fmt::Display`], but with colors.
 trait ColorDisplay {
     fn to_color_string(&self) -> String;
