@@ -16,7 +16,7 @@ use uv_pypi_types::Requirement;
 use uv_python::PythonEnvironment;
 use uv_settings::ToolOptions;
 use uv_shell::Shell;
-use uv_tool::{entrypoint_paths, find_executable_directory, InstalledTools, Tool, ToolEntrypoint};
+use uv_tool::{entrypoint_paths, tool_executable_dir, InstalledTools, Tool, ToolEntrypoint};
 use uv_warnings::warn_user;
 
 use crate::commands::ExitStatus;
@@ -79,7 +79,7 @@ pub(crate) fn install_executables(
     };
 
     // Find a suitable path to install into
-    let executable_directory = find_executable_directory()?;
+    let executable_directory = tool_executable_dir()?;
     fs_err::create_dir_all(&executable_directory)
         .context("Failed to create executable directory")?;
 
