@@ -4,12 +4,12 @@ use owo_colors::OwoColorize;
 
 use uv_configuration::PreviewMode;
 use uv_fs::Simplified;
-use uv_tool::{find_executable_directory, InstalledTools};
+use uv_tool::{tool_executable_dir, InstalledTools};
 
 /// Show the tool directory.
 pub(crate) fn dir(bin: bool, _preview: PreviewMode) -> anyhow::Result<()> {
     if bin {
-        let executable_directory = find_executable_directory()?;
+        let executable_directory = tool_executable_dir()?;
         println!("{}", executable_directory.simplified_display().cyan());
     } else {
         let installed_tools =
