@@ -24,6 +24,7 @@ use uv_python::{
     EnvironmentPreference, PythonDownloads, PythonInstallation, PythonPreference, PythonRequest,
 };
 use uv_resolver::{ExcludeNewer, FlatIndex};
+use uv_settings::PythonInstallMirrors;
 use uv_shell::Shell;
 use uv_types::{BuildContext, BuildIsolation, HashStrategy};
 use uv_warnings::{warn_user, warn_user_once};
@@ -42,6 +43,7 @@ pub(crate) async fn venv(
     project_dir: &Path,
     path: Option<PathBuf>,
     python_request: Option<&str>,
+    install_mirrors: PythonInstallMirrors,
     python_preference: PythonPreference,
     python_downloads: PythonDownloads,
     link_mode: LinkMode,
@@ -68,6 +70,7 @@ pub(crate) async fn venv(
         project_dir,
         path,
         python_request,
+        install_mirrors,
         link_mode,
         index_locations,
         index_strategy,
@@ -125,6 +128,7 @@ async fn venv_impl(
     project_dir: &Path,
     path: Option<PathBuf>,
     python_request: Option<&str>,
+    install_mirrors: PythonInstallMirrors,
     link_mode: LinkMode,
     index_locations: &IndexLocations,
     index_strategy: IndexStrategy,
@@ -205,6 +209,8 @@ async fn venv_impl(
         &client_builder,
         cache,
         Some(&reporter),
+        install_mirrors.python_install_mirror,
+        install_mirrors.pypy_install_mirror,
     )
     .await
     .into_diagnostic()?;
