@@ -7,11 +7,11 @@ use indoc::indoc;
 use insta::assert_snapshot;
 use predicates::str::contains;
 use std::path::Path;
-
+use uv_fs::copy_dir_all;
 use uv_python::PYTHON_VERSION_FILENAME;
 use uv_static::EnvVars;
 
-use crate::common::{copy_dir_all, uv_snapshot, TestContext};
+use crate::common::{uv_snapshot, TestContext};
 
 #[test]
 fn run_with_python_version() -> Result<()> {
@@ -248,7 +248,7 @@ fn run_no_args() -> Result<()> {
 
     The following commands are available in the environment:
 
-    - pydoc
+    - pydoc.bat
     - python
     - pythonw
 
@@ -1105,7 +1105,7 @@ fn run_with_editable() -> Result<()> {
     ----- stderr -----
     Resolved 3 packages in [TIME]
     Audited 3 packages in [TIME]
-      × Invalid `--with` requirement
+      × Failed to resolve `--with` requirement
       ╰─▶ Distribution not found at: file://[TEMP_DIR]/foo
     "###);
 
